/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! COPYRIGHT    (c)  2005 CAHA, Spain
! IDENT        trace_peaks_recursive.c
! LANGUAGE     C
! AUTHOR       S.F.Sanchez
! KEYWORDS     
! PURPOSE      C version of trace_peaks_recursive.pl
! COMMENT      No GROUP information included. No header data properly copied
! VERSION      0.1  08.08.05 : Creation, SF.
------------------------------------------------------------------------------*/

#include <IFU_io.h>

#include "stats.h"
#include <stdlib.h>


#include <stdio.h> //For Standard I/O
#include <ctype.h> //For Type check
#include <string.h> //For String manipulation
#include <math.h> //For Math functions
#include <data_io.h> //For Lyon I/O data

#include <cpgplot.h> // PGPLOT
#include "my_math.h"

main(int argc, char *ARGV[])
{

  
    IMAGE2D image,trace;
    char infile[80],trace_file[80],peaks_file[80];

    float y_shift=0;
    float plot_width;
    int width,spec_axis,y_width,y_width2,plot,nplot,nsearch,npeaks,flag;
    FILE *FH;

    int *id,*peak_y_pixel_ini,*peak_y_pixel;
    float *peak_y_max;
    int *peak_y_pixel_new;
    float *peak_y_max_new;

    long npix[2];
    double start[2];
    double step[2];
    char **argval, **arglabel;
    char car;
    int i,j,k,mean_point,mm;
    float min,max,peak_y_pre;
    float *sec_array,*cut;
    int existe_peak,peak;
    int a,b,c;
    float fa,fb,fc,den;
    float *x,*y;
    int nx,ny;
    float med,top,bottom;

    if (argc<10) {
	printf("USE: trace_peaks_recursive.pl RAW.fits Spectral_axis[0/1] PEAKS_FILE coadd_width search_width plot nplot nsearch TRACE.fits [y_shift] [2nd_search_width]\n");
	return(0);
    }

    set_user_dataformat();
    //init_session(ARGV,argc,&arglabel,&argval);


    strcpy(infile,ARGV[1]);
    spec_axis=atoi(ARGV[2]);
    strcpy(peaks_file,ARGV[3]);
    width=atoi(ARGV[4]);
    if (width==0) {
	width=1;
    }
    y_width=atoi(ARGV[5]);
    plot=atoi(ARGV[6]);
    nplot=atoi(ARGV[7]);
    nsearch=atoi(ARGV[8]);
    strcpy(trace_file,ARGV[9]);

    if (argc==11) {
	y_shift=atof(ARGV[10]);
    }
    y_width2=y_width;
    if (argc==12) {
	y_shift=atof(ARGV[10]);
	y_width2=atof(ARGV[11]);
    }


    printf("Reading peaks_file %s...",peaks_file);
    npeaks=0;

    if ((FH=fopen(peaks_file,"r"))==NULL) {
      printf("Error opening file: %s\n",peaks_file);
      return(-1);
    } else {    
	npeaks=0;
	while((car=getc(FH))!=EOF) {
	    flag=1;
	    if (car=='\n')
	    {
		npeaks+=1;
		flag+=1;
	    }
	}
	if (flag==1) {
	    npeaks=npeaks;
	} else {
	    npeaks=npeaks-1;
	}
	rewind(FH);
	// We jump one line
	flag=0;
	while(flag==0) {
	    car=getc(FH);
	    if (car=='\n') {
		 flag=1;
	    }
	}
	id = (int *)malloc((npeaks)*sizeof(int));  	
	peak_y_pixel_ini = (int *)malloc((npeaks)*sizeof(int));  	
	peak_y_pixel = (int *)malloc((npeaks)*sizeof(int));  	
	peak_y_max = (float *)malloc((npeaks)*sizeof(float));  	
	for (j=0;j<npeaks;j++) {
	    fscanf(FH, "%d", &id[j]);
	    fscanf(FH, "%d", &peak_y_pixel[j]);
	    fscanf(FH, "%f", &peak_y_max[j]);
	    peak_y_pixel[j]=peak_y_pixel[j]+y_shift;
	    peak_y_max[j]=peak_y_max[j]+y_shift;
	    peak_y_pixel_ini[j]=floor(peak_y_max[j]);
	    peak_y_pixel[j]=floor(peak_y_max[j]);
	}
    }



//    printf("DONE\n");
    printf("NPEAKS %d FOUND\n",npeaks);

    /*
     * We read the input data:
     */
    printf("Reading file infile '%s'",infile);
    open_frame(&image,infile,"I");
    if (spec_axis==0) {
	npix[0]=image.nx;
	npix[1]=npeaks;
	nx=image.nx;
	ny=image.ny;
    } else {
	npix[0]=image.ny;
	npix[1]=npeaks;
	nx=image.ny;
	ny=image.nx;
    }
    start[0]=0.0;
    start[1]=0.0;
    step[0]=1.0;
    step[1]=1.0;

/*
 * We start to trace;
 */

    sec_array = (float *)malloc((ny)*sizeof(float));  	
    cut = (float *)malloc((ny)*sizeof(float));  	
    peak_y_pixel_new  = (int *)malloc((npix[0]*npix[1])*sizeof(int));  	
    peak_y_max_new  = (float *)malloc((npix[0]*npix[1])*sizeof(float));  	
    mean_point=floor(npix[0]/2);

    printf("1st Half\n");
    for (mm=mean_point;mm<(nx-width);mm++) {	
	min=1e12;
	max=-1e12;
	for (j=0;j<ny;j++) {
	    sec_array[j]=0;
	    if (width>0) {
		for (i=(mm-width);i<(mm+width);i++) {
		    if (spec_axis==0) {
			sec_array[j]=sec_array[j]+RD_frame(&image,i,j);
		    } else {
			sec_array[j]=sec_array[j]+RD_frame(&image,j,i);
		    }
		}
	    } else {
		for (i=mm;i<(mm+1);i++) {
		    if (spec_axis==0) {
			sec_array[j]=sec_array[j]+RD_frame(&image,i,j);
		    } else {
			sec_array[j]=sec_array[j]+RD_frame(&image,j,i);
		    }
		}
	    }
	    if (min>sec_array[j]) {
		min=sec_array[j];
	    }
	    if (max<sec_array[j]) {
		max=sec_array[j];
	    }	    
	    cut[j]=1.0*j;
	}

	for (k=0;k<npeaks;k++) {
	    existe_peak=0;
	    if (mm==mean_point) {
		peak_y_pre=peak_y_pixel[k];
	    } else {
		peak_y_pre=peak_y_pixel_new[(mm-1)+k*nx];
		y_width=y_width2;
	    }
	    for (j=(peak_y_pre-y_width);j<(peak_y_pre+y_width);j++) {
		peak=1;
		for (i=0;i<nsearch;i++) {
		    if (sec_array[j-i]<sec_array[j-i-1]) {
			peak=0;
		    }
		    if (sec_array[j+i]<sec_array[j+i+1]) {
			peak=0;
		    }	       
		}
		if (peak==1) {
		    peak_y_pixel_new[mm+k*nx]=j*1.0;		
		    existe_peak=1;
		} 
		
	    }
	    if (existe_peak==0) {
		if (mm==mean_point) {
		    peak_y_pixel_new[mm+k*nx]=peak_y_pixel[k]*1.0;	
		    peak_y_max_new[mm+k*nx]=peak_y_max[k];		
		} else {
		    peak_y_pixel_new[mm+k*nx]=peak_y_pixel_new[mm-1+k*nx];
		    peak_y_max_new[mm+k*nx]=peak_y_max_new[mm-1+k*nx];
		}
	    } else {
		j=peak_y_pixel_new[mm+k*nx];		
		a=j-1;
		b=j;
		c=j+1;
		fa=-sec_array[a];
		fb=-sec_array[b];
		fc=-sec_array[c];
		den=(fc-2*fb+fa);
		if (den!=0) {
		    peak_y_max_new[mm+k*nx]=c-(b-a)*((fc-fb)/den+0.5);
		} else {
		    peak_y_max_new[mm+k*nx]=peak_y_max[k];		
		}
	    }
	    peak_y_pixel[k]=peak_y_pixel_new[mm+k*nx];
	}
	if (plot==1) {

	  med=fmedian(sec_array,ny);
	  bottom=(-1)*med;
	  top=10*med;
	  if (max>top) {
	    max=top;
	  }
	  if (min<bottom) {
	    min=bottom;
	  }

	  //	  printf("med=%f bot=%f top=%f max=%f min=%f\n",med,bottom,top,max,min);
	  //	  ******
	  //	    cpgbegin(0,"/xs",1,1);
	    cpgopen("/xs");
	    cpgask(0);
	    cpgsfs(1.2);
	    cpgscf(2);             
	    cpgslw(2);
	    cpgsch(1.2);
	    cpgsubp(1,nplot);
	    x = (float *)malloc((npeaks)*sizeof(float));  	
	    y = (float *)malloc((npeaks)*sizeof(float));  	
	    for (i=0;i<nplot;i++) {
		cpgenv((ny/nplot)*i,(ny/nplot)*(i+1),min,max,0,0);
		cpglab("Y-axis","Counts","");
		cpgline(ny,cut,sec_array);    
		cpgsch(4.0);           
		cpgsci(7);
		for (k=0;k<npeaks;k++) {
		    x[k]=peak_y_max[k];
		    y[k]=0.8*(max-min)+min;
		}		
		cpgpt(npeaks,x,y,1);
		cpgsci(2);
		for (k=0;k<npeaks;k++) {
		    x[k]=peak_y_pixel_new[mm+k*nx];
		    y[k]=0.7*(max-min)+min;
		}		
		cpgpt(npeaks,x,y,5);
		cpgsci(4);
		for (k=0;k<npeaks;k++) {
		    x[k]=peak_y_max_new[mm+k*nx];
		    y[k]=0.6*(max-min)+min;		
		}
		cpgpt(npeaks,x,y,2);
		cpgsci(1);
		cpgsch(1.2);           
	    }
	    free(x);
	    free(y);
	    cpgsci(1);
	    cpgclos();
	    cpgend;
	    if (car!='A') {
		printf("Press Enter (A for automatic, 0 for no plot, N for next plot):\n"); 
		car=getc(stdin);
	    }
	    if (car=='0') {
		plot=0;
	    }
	    if (car=='N') {
		plot=2;
	    }
	}

	if (mm==100*ceil(mm/100)) {
	    printf("(1st) %d/%d\n",mm,nx);
	}
    } 


    printf("2nd Half\n");
    for (mm=mean_point-1;mm>width;mm--) {
	min=1e12;
	max=-1e12;
	for (j=0;j<ny;j++) {
	    sec_array[j]=0;
	    if (width>0) {
		for (i=(mm-width);i<(mm+width);i++) {
		    if (spec_axis==0) {
			sec_array[j]=sec_array[j]+RD_frame(&image,i,j);
		    } else {
			sec_array[j]=sec_array[j]+RD_frame(&image,j,i);
		    }
		}
	    } else {
		for (i=mm;i<(mm+1);i++) {
		    if (spec_axis==0) {
			sec_array[j]=sec_array[j]+RD_frame(&image,i,j);
		    } else {
			sec_array[j]=sec_array[j]+RD_frame(&image,j,i);
		    }
		}
	    }
	    if (min>sec_array[j]) {
		min=sec_array[j];
	    }
	    if (max<sec_array[j]) {
		max=sec_array[j];
	    }	    
	    cut[j]=1.0*j;
	}

	for (k=0;k<npeaks;k++) {
	    existe_peak=0;
	    if (mm==mean_point) {
		peak_y_pre=peak_y_pixel[k];
	    } else {
		peak_y_pre=peak_y_pixel_new[(mm+1)+k*nx];
	    }
	    for (j=(peak_y_pre-y_width);j<(peak_y_pre+y_width);j++) {
		peak=1;
		for (i=0;i<nsearch;i++) {
		    if (sec_array[j-i]<sec_array[j-i-1]) {
			peak=0;
		    }
		    if (sec_array[j+i]<sec_array[j+i+1]) {
			peak=0;
		    }	       
		}
		if (peak==1) {
		    peak_y_pixel_new[mm+k*nx]=j*1.0;		
		    existe_peak=1;
		} 
		
	    }
	    if (existe_peak==0) {
		if (mm==mean_point) {
		    peak_y_pixel_new[mm+k*nx]=peak_y_pixel[k]*1.0;	
		    peak_y_max_new[mm+k*nx]=peak_y_max[k];		
		} else {
		    peak_y_pixel_new[mm+k*nx]=peak_y_pixel_new[mm+1+k*nx];
		    peak_y_max_new[mm+k*nx]=peak_y_max_new[mm+1+k*nx];
		}
	    } else {
		j=peak_y_pixel_new[mm+k*nx];		
		a=j-1;
		b=j;
		c=j+1;
		fa=-sec_array[a];
		fb=-sec_array[b];
		fc=-sec_array[c];
		den=(fc-2*fb+fa);
		if (den!=0) {
		    peak_y_max_new[mm+k*nx]=c-(b-a)*((fc-fb)/den+0.5);
		} else {
		    peak_y_max_new[mm+k*nx]=peak_y_max[k];		
		}
	    }
	    peak_y_pixel[k]=peak_y_pixel_new[mm+k*nx];
	}
	if (plot==1) {
	  med=fmedian(sec_array,ny);
	  bottom=(-1)*med;
	  top=10*med;
	  if (max>top) {
	    max=top;
	  }
	  if (min<bottom) {
	    min=bottom;
	  }

//	    cpgbegin(0,"/xs",1,1);
	    cpgopen("/xs");
	    cpgask(0);
	    cpgsfs(1.2);
	    cpgscf(2);             
	    cpgslw(2);
	    cpgsch(1.2);
	    cpgsubp(1,nplot);
	    x = (float *)malloc((npeaks)*sizeof(float));  	
	    y = (float *)malloc((npeaks)*sizeof(float));  	
	    for (i=0;i<nplot;i++) {
		cpgenv((ny/nplot)*i,(ny/nplot)*(i+1),min,max,0,0);
		cpglab("Y-axis","Counts","");
		cpgline(ny,cut,sec_array);    
		cpgsch(4.0);           
		cpgsci(7);
		for (k=0;k<npeaks;k++) {
		    x[k]=peak_y_max[k];
		    y[k]=0.8*(max-min)+min;
		}		
		cpgpt(npeaks,x,y,1);
		cpgsci(2);
		for (k=0;k<npeaks;k++) {
		    x[k]=peak_y_pixel_new[mm+k*nx];
		    y[k]=0.7*(max-min)+min;
		}		
		cpgpt(npeaks,x,y,5);
		cpgsci(4);
		for (k=0;k<npeaks;k++) {
		    x[k]=peak_y_max_new[mm+k*nx];
		    y[k]=0.6*(max-min)+min;		
		}
		cpgpt(npeaks,x,y,2);
		cpgsci(1);
		cpgsch(1.2);           
	    }
	    free(x);
	    free(y);
	    cpgsci(1);
	    cpgclos();
	    cpgend;
	    if (car!='A') {
		printf("Press Enter (A for automatic, 0 for no plot, N for next plot):\n"); 
		car=getc(stdin);
	    }
	    if (car=='0') {
		plot=0;
	    }
	    if (car=='N') {
		plot=2;
	    }
	}

	if (mm==100*ceil(mm/100)) {
	    printf("(1st) %d/%d\n",mm,nx);
	}
    } 
    close_frame(&image);    


    for (mm=0;mm<width+1;mm++) {
	for (k=0;k<npeaks;k++) {
	    peak_y_pixel_new[mm+k*nx]=peak_y_pixel_new[width+1+k*nx];
	    peak_y_max_new[mm+k*nx]=peak_y_max_new[width+1+k*nx];
	}
    }
    for (mm=(nx-width);mm<nx;mm++) {
	for (k=0;k<npeaks;k++) {
	    peak_y_pixel_new[mm+k*nx]=peak_y_pixel_new[nx-width-1+k*nx];
	    peak_y_max_new[mm+k*nx]=peak_y_max_new[nx-width-1+k*nx];
	}
    }



 if ((plot==1)||(plot==2)) {
     cpgopen("/xs");
     cpgask(0);
     cpgsfs(1.2);
     cpgscf(2);             
     cpgslw(2);
     cpgsch(1.2);
     for (k=0;k<npeaks;k++) {
	 //cpgenv(0,nx,peak_y_pixel_ini[k]-2*y_width-y_width2,peak_y_pixel_ini[k]+y_width2+2*y_width,0,0);
	 max=-1e12;
	 min=1e12;
	for (i=0;i<nx;i++) {
	    if ((peak_y_max_new[i+k*nx]<min)&&(peak_y_max_new[i+k*nx]!=0)) {
		min=peak_y_max_new[i+k*nx];
	    }
	    if (peak_y_max_new[i+k*nx]>max) {
		max=peak_y_max_new[i+k*nx];
	    }
	}
	cpgenv(0,nx,min-1,max+1,0,0);
	cpglab("X-axis","Y-Axis","");
	printf("%d/%d\n",k,npeaks);
	cpgsci(7);
	x = (float *)malloc((nx)*sizeof(float));  	
	y = (float *)malloc((nx)*sizeof(float));  	
	for (i=0;i<nx;i++) {
	    x[i]=1.0*i;
	    y[i]=peak_y_pixel[k];
	}
	cpgpt(nx,x,y,1);
	cpgsci(2);
	for (i=0;i<nx;i++) {
	    x[i]=1.0*i;
	    y[i]=peak_y_pixel_new[i+k*nx];
	}
	cpgpt(nx,x,y,1);
	cpgsci(4);
	for (i=0;i<nx;i++) {
	    x[i]=1.0*i;
	    y[i]=peak_y_max_new[i+k*nx];
	}
	cpgpt(nx,x,y,1);
	cpgsci(1);
     if (car!='A') {
	 printf("Press Enter (A for automatic, 0 for no plot, N for next plot):\n"); 
	 car=getc(stdin);
     }
     if (car=='0') {
	 plot=0;
     }
     if (car=='N') {
	 plot=3;
	 k=npeaks;
     }	
     }
     cpgclos;
     cpgend;


 }




 printf("Writting the tracing solution: %s\n",trace_file);
    create_frame(&trace,trace_file,npix,start,step,FLOAT,"EXTRACTED",NULL);
    for (i=0;i<npix[0];i++) {
	for (j=0;j<npix[1];j++) {
	    WR_frame(&trace,i,j,peak_y_max_new[i+j*nx]);
	}
    }
    close_frame(&trace);    

    printf("DONE\n");
//    exit_session(0);
    return(0);	
}
